/* wmudmount
 * Copyright © 2010  Brad Jorsch <anomie@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef UDISKS_H
#define UDISKS_H

#include <dbus/dbus-glib.h>
#include <gio/gio.h>

/* Types */

#define UDISKS_FILESYSTEM 1
#define UDISKS_LUKS 2

/* Structs */

struct udisks_fs_info {
    char *id;
    char *name;
    gboolean is_known;
    //gboolean supports_unix_owners;
    gboolean can_mount;
    //gboolean can_create;
    guint32 max_label_len;
    gboolean supports_label_rename;
    gboolean supports_online_label_rename;
    //gboolean supports_fsck;
    //gboolean supports_online_fsck;
    //gboolean supports_resize_enlarge;
    //gboolean supports_online_resize_enlarge;
    //gboolean supports_resize_shrink;
    //gboolean supports_online_resize_shrink;
};

typedef struct _udisks_volume {
    int type;
    const char *path;
    const char *native_path;
    const char *label;
    const char *old_label;
    const char *desc;
    const char *uuid;
    const char *dev;
    GIcon *icon, *unemblemed_icon;
    int pending_actions;

    struct _udisks_volume *parent;

    union {
        struct {
            const char *mountpoint;
            guint64 total_size, free_size;
            const struct udisks_fs_info *info;
        } fs;
        struct {
            // All devices wanting to support udisks_unlock must begin with
            // these two fields:
            const char *device_type; // identifier, e.g. "LUKS"
            const char *device_uuid; // IdUuid only, unlike v->uuid
        } _generic_crypto;
        struct {
            const char *device_type;
            const char *device_uuid;
        } luks;
    } d;

    int root_only:1;
    int readonly:1;
    int removable:1;

    int force_include;
    int force_exclude;
    int include:1;

    /* private */
    int can_unlock:1;
    int can_lock:1;
    int can_eject:1;
    int can_detach:1;
    int can_mount:1;
    int can_relabel:1;
    int refct;
    DBusGProxy *proxy;
} udisks_volume;

/* Variables */

extern const char *udisks_daemon;
extern int num_volumes;
extern udisks_volume **Volumes;
extern gboolean fsm_blink;
extern gboolean blink_full_filesystems;

/* Functions */

typedef void (*udisks_callback)(udisks_volume *v, gpointer ret, gpointer data, gboolean failed, GError *err);

#define udisks_get_display_name(v) ((v)->label?(v)->label:(v)->old_label?(v)->old_label:(v)->dev)
#define udisks_display_name_should_right_justify(v) (!(v)->label && !(v)->old_label)

void udisks_ref(udisks_volume *v);
void udisks_unref(udisks_volume *v);

static inline gboolean udisks_can_unlock(udisks_volume *v){
    for(; v; v=v->parent){
        if(v->can_unlock) return TRUE;
    }
    return FALSE;
}
static inline gboolean udisks_can_lock(udisks_volume *v){
    for(; v; v=v->parent){
        if(v->can_lock) return TRUE;
    }
    return FALSE;
}
static inline gboolean udisks_can_eject(udisks_volume *v){
    for(; v; v=v->parent){
        if(v->can_eject) return TRUE;
    }
    return FALSE;
}
static inline gboolean udisks_can_detach(udisks_volume *v){
    for(; v; v=v->parent){
        if(v->can_detach) return TRUE;
    }
    return FALSE;
}
static inline gboolean udisks_can_mount(udisks_volume *v){
    return v->can_mount?TRUE:FALSE;
}
static inline gboolean udisks_can_relabel(udisks_volume *v){
    return v->can_relabel?TRUE:FALSE;
}

gboolean udisks_init();
gboolean udisks_mount(udisks_volume *v, udisks_callback callback, gpointer data);
gboolean udisks_unmount(udisks_volume *v, udisks_callback callback, gpointer data);
gboolean udisks_eject(udisks_volume *v, udisks_callback callback, gpointer data);
gboolean udisks_detach(udisks_volume *v, udisks_callback callback, gpointer data);
gboolean udisks_lock(udisks_volume *v, udisks_callback callback, gpointer data);
gboolean udisks_unlock(udisks_volume *v, udisks_callback callback, gpointer data);
gboolean udisks_forget_passphrase(udisks_volume *v, udisks_callback callback, gpointer data);
gboolean udisks_relabel(udisks_volume *v, udisks_callback callback, gpointer data);

#endif
